@@ -20,42 +20,44 @@ import aaargh
 import grandfatherson
 from byteformat import ByteFormatter
 
-from bakthat.backends import GlacierBackend, S3Backend, RotationConfig
+from bakthat.backends import GlacierBackend, S3Backend, RotationConfig, SwiftBackend
 from bakthat.conf import config, DEFAULT_DESTINATION, DEFAULT_LOCATION, CONFIG_FILE
 from bakthat.utils import _interval_string_to_seconds
-from bakthat.models import Backups, Inventory
-from bakthat.sync import BakSyncer
+from bakthat.models import Backups
+from bakthat.sync import BakSyncer, bakmanager_hook
 
-__version__ = "0.4.5"
+__version__ = "0.5.0"
 
-app = aaargh.App(description="Compress, encrypt and upload files directly to Amazon S3/Glacier.")
+app = aaargh.App(description="Compress, encrypt and upload files directly to Amazon S3/Glacier/Swift.")
 
 log = logging.getLogger()
 
 if not log.handlers:
     logging.basicConfig(level=logging.INFO, format='%(message)s')
 
-STORAGE_BACKEND = dict(s3=S3Backend, glacier=GlacierBackend)
+STORAGE_BACKEND = dict(s3=S3Backend, glacier=GlacierBackend, swift=SwiftBackend)
 
 
-def _get_store_backend(conf, destination=DEFAULT_DESTINATION, profile="default"):
+def _get_store_backend(conf, destination=None, profile="default"):
+    if not conf:  # If not conf is defined, we load the profile conf
+        conf = config.get(profile)
     if not destination:
-        destination = config.get("aws", "default_destination")
-    return STORAGE_BACKEND[destination](conf, profile)
+        destination = conf.get("default_destination", DEFAULT_DESTINATION)
+    return STORAGE_BACKEND[destination](conf, profile), destination, conf
 
 
-def _match_filename(filename, destination=DEFAULT_DESTINATION, conf=None, profile="default"):
+def _match_filename(filename, destination=None, conf=None, profile="default"):
     """Return all stored backups keys for a given filename."""
     if not filename:
         raise Exception("Filename can't be blank")
-    storage_backend = _get_store_backend(conf, destination, profile)
+    storage_backend, destination, conf = _get_store_backend(conf, destination, profile)
 
     keys = [name for name in storage_backend.ls() if name.startswith(filename)]
     keys.sort(reverse=True)
     return keys
 
 
-def match_filename(filename, destination=DEFAULT_DESTINATION, conf=None, profile="default"):
+def match_filename(filename, destination=None, conf=None, profile="default"):
     """Return a list of dict with backup_name, date_component, and is_enc."""
     _keys = _match_filename(filename, destination, conf, profile)
     regex_key = re.compile(r"(?P<backup_name>.+)\.(?P<date_component>\d{14})\.tgz(?P<is_enc>\.enc)?")
@@ -82,9 +84,9 @@ def match_filename(filename, destination=DEFAULT_DESTINATION, conf=None, profile
 @app.cmd(help="Delete backups older than the given interval string.")
 @app.cmd_arg('filename', type=str, help="Filename to delete")
 @app.cmd_arg('interval', type=str, help="Interval string like 1M, 1W, 1M3W4h2s")
-@app.cmd_arg('-d', '--destination', type=str, help="s3|glacier")
+@app.cmd_arg('-d', '--destination', type=str, help="s3|glacier|swift", default=None)
 @app.cmd_arg('-p', '--profile', type=str, default="default", help="profile name (default by default)")
-def delete_older_than(filename, interval, destination=DEFAULT_DESTINATION, profile="default", **kwargs):
+def delete_older_than(filename, interval, destination=None, profile="default", **kwargs):
     """Delete backups matching the given filename older than the given interval string.
 
     :type filename: str
@@ -95,7 +97,7 @@ def delete_older_than(filename, interval, destination=DEFAULT_DESTINATION, profi
         (s => seconds, m => minutes, h => hours, D => days, W => weeks, M => months, Y => Years).
 
     :type destination: str
-    :param destination: glacier|s3
+    :param destination: glacier|s3|swift
 
     :type conf: dict
     :keyword conf: Override/set AWS configuration.
@@ -105,7 +107,7 @@ def delete_older_than(filename, interval, destination=DEFAULT_DESTINATION, profi
 
     """
     conf = kwargs.get("conf")
-    storage_backend = _get_store_backend(conf, destination, profile)
+    storage_backend, destination, conf = _get_store_backend(conf, destination, profile)
     interval_seconds = _interval_string_to_seconds(interval)
 
     deleted = []
@@ -126,16 +128,16 @@ def delete_older_than(filename, interval, destination=DEFAULT_DESTINATION, profi
 
 @app.cmd(help="Rotate backups using Grandfather-father-son backup rotation scheme.")
 @app.cmd_arg('filename', type=str)
-@app.cmd_arg('-d', '--destination', type=str, help="s3|glacier", default=DEFAULT_DESTINATION)
+@app.cmd_arg('-d', '--destination', type=str, help="s3|glacier|swift", default=None)
 @app.cmd_arg('-p', '--profile', type=str, default="default", help="profile name (default by default)")
-def rotate_backups(filename, destination=DEFAULT_DESTINATION, profile="default", **kwargs):
+def rotate_backups(filename, destination=None, profile="default", **kwargs):
     """Rotate backup using grandfather-father-son rotation scheme.
 
     :type filename: str
     :param filename: File/directory name.
 
     :type destination: str
-    :param destination: s3|glacier
+    :param destination: s3|glacier|swift
 
     :type conf: dict
     :keyword conf: Override/set AWS configuration.
@@ -157,7 +159,7 @@ def rotate_backups(filename, destination=DEFAULT_DESTINATION, profile="default",
 
     """
     conf = kwargs.get("conf", None)
-    storage_backend = _get_store_backend(conf, destination, profile)
+    storage_backend, destination, conf = _get_store_backend(conf, destination, profile)
     rotate = RotationConfig(conf, profile)
     if not rotate:
         raise Exception("You must run bakthat configure_backups_rotation or provide rotation configuration.")
@@ -192,18 +194,19 @@ def rotate_backups(filename, destination=DEFAULT_DESTINATION, profile="default",
 
 @app.cmd(help="Backup a file or a directory, backup the current directory if no arg is provided.")
 @app.cmd_arg('filename', type=str, default=os.getcwd(), nargs="?")
-@app.cmd_arg('-d', '--destination', type=str, help="s3|glacier", default=DEFAULT_DESTINATION)
+@app.cmd_arg('-d', '--destination', type=str, help="s3|glacier|swift", default=None)
 @app.cmd_arg('--prompt', type=str, help="yes|no", default="yes")
 @app.cmd_arg('-t', '--tags', type=str, help="space separated tags", default="")
 @app.cmd_arg('-p', '--profile', type=str, default="default", help="profile name (default by default)")
-def backup(filename=os.getcwd(), destination=None, prompt="yes", tags=[], profile="default", **kwargs):
+@app.cmd_arg('-k', '--key', type=str, default=None, help="Custom key for periodic backups (works only with BakManager.io hook.)")
+def backup(filename=os.getcwd(), destination=None, prompt="yes", tags=[], profile="default", key=None, **kwargs):
     """Perform backup.
 
     :type filename: str
     :param filename: File/directory to backup.
 
     :type destination: str
-    :param destination: s3|glacier
+    :param destination: s3|glacier|swift
 
     :type prompt: str
     :param prompt: Disable password promp, disable encryption,
@@ -227,9 +230,18 @@ def backup(filename=os.getcwd(), destination=None, prompt="yes", tags=[], profil
 
     """
     conf = kwargs.get("conf", None)
-    storage_backend = _get_store_backend(conf, destination, profile)
+    storage_backend, destination, conf = _get_store_backend(conf, destination, profile)
     backup_file_fmt = "{0}.{1}.tgz"
 
+    # Check if compression is disabled on the configuration.
+    if conf:
+        compress = conf.get("compress", True)
+    else:
+        compress = config.get(profile).get("compress", True)
+
+    if not compress:
+        backup_file_fmt = "{0}.{1}"
+
     log.info("Backing up " + filename)
     arcname = filename.strip('/').split('/')[-1]
     now = datetime.utcnow()
@@ -243,6 +255,9 @@ def backup(filename=os.getcwd(), destination=None, prompt="yes", tags=[], profil
                        backend=destination,
                        is_deleted=False)
 
+    # Useful only when using bakmanager.io hook
+    backup_key = key
+
     password = kwargs.get("password")
     if password is None and prompt.lower() != "no":
         password = getpass("Password (blank to disable encryption): ")
@@ -252,8 +267,15 @@ def backup(filename=os.getcwd(), destination=None, prompt="yes", tags=[], profil
                 log.error("Password confirmation doesn't match")
                 return
 
+    if not compress:
+        log.info("Compression disabled")
+        outname = filename
+        with open(outname) as outfile:
+            backup_data["size"] = os.fstat(outfile.fileno()).st_size
+        bakthat_compression = False
+
     # Check if the file is not already compressed
-    if mimetypes.guess_type(arcname) == ('application/x-tar', 'gzip'):
+    elif mimetypes.guess_type(arcname) == ('application/x-tar', 'gzip'):
         log.info("File already compressed")
         outname = filename
 
@@ -311,7 +333,7 @@ def backup(filename=os.getcwd(), destination=None, prompt="yes", tags=[], profil
     storage_backend.upload(stored_filename, outname)
 
     # We only remove the file if the archive is created by bakthat
-    if bakthat_encryption:
+    if bakthat_compression or bakthat_encryption:
         os.remove(outname)
 
     log.debug(backup_data)
@@ -321,16 +343,20 @@ def backup(filename=os.getcwd(), destination=None, prompt="yes", tags=[], profil
 
     BakSyncer(conf).sync_auto()
 
+    # bakmanager.io hook, enable with -k/--key paramter
+    if backup_key:
+        bakmanager_hook(conf, backup_data, backup_key)
+
     return backup_data
 
 
 @app.cmd(help="Give informations about stored filename, current directory if no arg is provided.")
 @app.cmd_arg('filename', type=str, default=os.getcwd(), nargs="?")
-@app.cmd_arg('-d', '--destination', type=str, help="s3|glacier")
+@app.cmd_arg('-d', '--destination', type=str, help="s3|glacier|swift", default=None)
 @app.cmd_arg('-p', '--profile', type=str, default="default", help="profile name (default by default)")
 def info(filename=os.getcwd(), destination=None, profile="default", **kwargs):
     conf = kwargs.get("conf", None)
-    storage_backend = _get_store_backend(conf, destination, profile)
+    storage_backend, destination, conf = _get_store_backend(conf, destination, profile)
     filename = filename.split("/")[-1]
     keys = match_filename(filename, destination if destination else DEFAULT_DESTINATION, profile)
     if not keys:
@@ -344,7 +370,7 @@ def info(filename=os.getcwd(), destination=None, profile="default", **kwargs):
 
 @app.cmd(help="Show backups list.")
 @app.cmd_arg('query', type=str, default="", help="search filename for query", nargs="?")
-@app.cmd_arg('-d', '--destination', type=str, default="", help="glacier|s3, default both")
+@app.cmd_arg('-d', '--destination', type=str, default="", help="glacier|s3|swift, show every destination by default")
 @app.cmd_arg('-t', '--tags', type=str, default="", help="tags space separated")
 @app.cmd_arg('-p', '--profile', type=str, default="default", help="profile name (all profiles are displayed by default)")
 def show(query="", destination="", tags="", profile="default", help="Profile, blank to show all"):
@@ -380,10 +406,10 @@ def configure(profile="default"):
             default_destination = raw_input("Default destination ({0}): ".format(DEFAULT_DESTINATION))
             if default_destination:
                 default_destination = default_destination.lower()
-                if default_destination in ("s3", "glacier"):
+                if default_destination in ("s3", "glacier", "swift"):
                     break
                 else:
-                    log.error("Invalid default_destination, should be s3 or glacier, try again.")
+                    log.error("Invalid default_destination, should be s3 or glacier, swift, try again.")
             else:
                 default_destination = DEFAULT_DESTINATION
                 break
@@ -394,6 +420,10 @@ def configure(profile="default"):
             region_name = DEFAULT_LOCATION
         new_conf[profile]["region_name"] = region_name
 
+        if default_destination in ("swift"):
+            new_conf[profile]["auth_version"] = raw_input("Swift Auth Version: ")
+            new_conf[profile]["auth_url"] = raw_input("Swift Auth URL: ")
+
         yaml.dump(new_conf, open(CONFIG_FILE, "w"), default_flow_style=False)
 
         log.info("Config written in %s" % CONFIG_FILE)
@@ -430,16 +460,16 @@ def configure_backups_rotation(profile="default"):
 
 @app.cmd(help="Restore backup in the current directory.")
 @app.cmd_arg('filename', type=str)
-@app.cmd_arg('-d', '--destination', type=str, help="s3|glacier", default=DEFAULT_DESTINATION)
+@app.cmd_arg('-d', '--destination', type=str, help="s3|glacier|swift", default=None)
 @app.cmd_arg('-p', '--profile', type=str, default="default", help="profile name (default by default)")
-def restore(filename, destination=DEFAULT_DESTINATION, profile="default", **kwargs):
+def restore(filename, destination=None, profile="default", **kwargs):
     """Restore backup in the current working directory.
 
     :type filename: str
     :param filename: File/directory to backup.
 
     :type destination: str
-    :param destination: s3|glacier
+    :param destination: s3|glacier|swift
 
     :type profile: str
     :param profile: Profile name (default by default).
@@ -451,7 +481,7 @@ def restore(filename, destination=DEFAULT_DESTINATION, profile="default", **kwar
     :return: True if successful.
     """
     conf = kwargs.get("conf", None)
-    storage_backend = _get_store_backend(conf, destination, profile)
+    storage_backend, destination, conf = _get_store_backend(conf, destination, profile)
 
     if not filename:
         log.error("No file to restore, use -f to specify one.")
@@ -491,7 +521,7 @@ def restore(filename, destination=DEFAULT_DESTINATION, profile="default", **kwar
         decrypt(out, decrypted_out, password)
         out = decrypted_out
 
-    if out:
+    if out and (key_name.endswith(".tgz") or key_name.endswith(".tgz.enc")):
         log.info("Uncompressing...")
         out.seek(0)
         if not backup.metadata.get("KeyValue"):
@@ -504,20 +534,27 @@ def restore(filename, destination=DEFAULT_DESTINATION, profile="default", **kwar
                     out.write(f.read())
 
         return True
+    elif out:
+        log.info("Backup is not compressed")
+        with open(backup.filename, "w") as restored:
+            out.seek(0)
+            restored.write(out.read())
+
+        return True
 
 
 @app.cmd(help="Delete a backup.")
 @app.cmd_arg('filename', type=str)
-@app.cmd_arg('-d', '--destination', type=str, help="s3|glacier", default=DEFAULT_DESTINATION)
+@app.cmd_arg('-d', '--destination', type=str, help="s3|glacier|swift", default=None)
 @app.cmd_arg('-p', '--profile', type=str, default="default", help="profile name (default by default)")
-def delete(filename, destination=DEFAULT_DESTINATION, profile="default", **kwargs):
+def delete(filename, destination=None, profile="default", **kwargs):
     """Delete a backup.
 
     :type filename: str
     :param filename: stored filename to delete.
 
     :type destination: str
-    :param destination: glacier|s3
+    :param destination: glacier|s3|swift
 
     :type profile: str
     :param profile: Profile name (default by default).
@@ -546,7 +583,7 @@ def delete(filename, destination=DEFAULT_DESTINATION, profile="default", **kwarg
 
     key_name = backup.stored_filename
 
-    storage_backend = _get_store_backend(conf, destination, profile)
+    storage_backend, destination, conf = _get_store_backend(conf, destination, profile)
 
     log.info("Deleting {0}".format(key_name))
 
@@ -572,23 +609,6 @@ def reset_sync(**kwargs):
     BakSyncer(conf).reset_sync()
 
 
-@app.cmd(help="List stored backups.")
-@app.cmd_arg('-d', '--destination', type=str, help="s3|glacier")
-@app.cmd_arg('-p', '--profile', type=str, default="default", help="profile name (default by default)")
-def ls(destination=None, profile="default", **kwargs):
-    conf = kwargs.get("conf", None)
-    storage_backend = _get_store_backend(conf, destination, profile)
-
-    log.info(storage_backend.container)
-
-    ls_result = storage_backend.ls()
-
-    for filename in ls_result:
-        log.info(filename)
-
-    return ls_result
-
-
 @app.cmd(help="Show Glacier inventory from S3")
 def show_glacier_inventory(**kwargs):
     if config.get("aws", "s3_bucket"):
@@ -636,56 +656,6 @@ def restore_glacier_inventory(**kwargs):
     glacier_backend.restore_inventory()
 
 
-@app.cmd()
-def upgrade_from_shelve():
-    if os.path.isfile(os.path.expanduser("~/.bakthat.db")):
-        glacier_backend = GlacierBackend()
-        glacier_backend.upgrade_from_shelve()
-
-        s3_backend = S3Backend()
-
-        regex_key = re.compile(r"(?P<backup_name>.+)\.(?P<date_component>\d{14})\.tgz(?P<is_enc>\.enc)?")
-
-        # old regex for backward compatibility (for files without dot before the date component).
-        old_regex_key = re.compile(r"(?P<backup_name>.+)(?P<date_component>\d{14})\.tgz(?P<is_enc>\.enc)?")
-
-        for generator, backend in [(s3_backend.ls(), "s3"), ([ivt.filename for ivt in Inventory.select()], "glacier")]:
-            for key in generator:
-                match = regex_key.match(key)
-                # Backward compatibility
-                if not match:
-                    match = old_regex_key.match(key)
-                if match:
-                    filename = match.group("backup_name")
-                    is_enc = bool(match.group("is_enc"))
-                    backup_date = int(datetime.strptime(match.group("date_component"), "%Y%m%d%H%M%S").strftime("%s"))
-                else:
-                    filename = key
-                    is_enc = False
-                    backup_date = 0
-                if backend == "s3":
-                    backend_hash = hashlib.sha512(s3_backend.conf.get("access_key") + \
-                                        s3_backend.conf.get(s3_backend.container_key)).hexdigest()
-                elif backend == "glacier":
-                    backend_hash = hashlib.sha512(glacier_backend.conf.get("access_key") + \
-                                        glacier_backend.conf.get(glacier_backend.container_key)).hexdigest()
-                new_backup = dict(backend=backend,
-                                  is_deleted=0,
-                                  backup_date=backup_date,
-                                  tags="",
-                                  stored_filename=key,
-                                  filename=filename,
-                                  last_updated=int(datetime.utcnow().strftime("%s")),
-                                  metadata=dict(is_enc=is_enc),
-                                  size=0,
-                                  backend_hash=backend_hash)
-                try:
-                    Backups.upsert(**new_backup)
-                except Exception, exc:
-                    print exc
-        os.remove(os.path.expanduser("~/.bakthat.db"))
-
-
 def main():
     app.run()
 
